open Base

module type S = sig
  (** Asks user for a password and a confirmation and returns the password *)
  val read_password: unit -> string

  (** Awaits confirmation of an action from user.
      Returns true if confirmation was given, false otherwise. *)
  val await_confirmation: string -> bool

  (** Prints string to the stdout and flushes it *)
  val print_string: string -> unit

  (** Prints string to the stdout, ends it with an endline and flushes *)
  val print_endline: string -> unit
end

module Real_world: S = struct
  let print_string str =
    Stdio.print_string str;
    Stdio.Out_channel.flush Stdio.stdout

  let print_endline =
    Stdio.print_endline

  let read_line_exn () =
    Stdio.In_channel.input_line_exn Stdio.stdin

  let read_password () =
    let read_password_once () =
      let open Core.Unix in
      let open Core.Unix.Terminal_io in
      let disable_echo () =
        let tios = tcgetattr stdin in
        tios.c_echo <- false;
        tcsetattr tios stdin ~mode:TCSANOW in
      let enable_echo () =
        let tios = tcgetattr stdin in
        tios.c_echo <- true;
        tcsetattr tios stdin ~mode:TCSANOW in
      let read_pw prompt =
        print_string prompt;
        disable_echo();
        let result = read_line_exn () in
        enable_echo();
        print_endline "";
        result in
      Exn.protect
        ~f:begin fun () ->
          let password = read_pw "Enter password: " in
          let confirmation = read_pw "Confirm password: " in
          match (String.equal password confirmation, String.length password >= 8) with
          | true, true ->
            Some password
          | false, _ ->
            print_endline "Passwords do not match. Please try again.";
            None
          | _, false ->
            print_endline "Password is too short. It must be at least 8 symbols long. Please try again.";
            None
        end
        ~finally:enable_echo in
    let opt_password = ref None in
    while Option.is_none !opt_password do
      opt_password := read_password_once ()
    done;
    Option.value_exn !opt_password

  let await_confirmation msg =
    print_endline msg;
    print_string "Type YES to confirm: ";
    let confirmation = read_line_exn () in
    String.equal confirmation "YES"
end
